@@ -1,34 +1,32 @@
-from copy import deepcopy
-from datetime import datetime
+import json
+import logging
 import os
 import sys
 import time
-import json
-import logging
+from copy import deepcopy
+from datetime import datetime
 from itertools import islice
-from typing import Dict, Optional, Tuple
+from typing import Dict, Optional, Tuple, Union, List, Set
 
+import click
 import rich
 from rich.logging import RichHandler
-import click
-
-from data_diff import Database
-from data_diff.schema import create_schema
-from data_diff.queries.api import current_timestamp
 
+from data_diff import Database, DbPath
+from data_diff.config import apply_config_from_file
+from data_diff.databases._connect import connect
 from data_diff.dbt import dbt_diff
-from data_diff.utils import eval_name_template, remove_password_from_url, safezip, match_like, LogStatusHandler
-from data_diff.diff_tables import Algorithm
+from data_diff.diff_tables import Algorithm, TableDiffer
 from data_diff.hashdiff_tables import HashDiffer, DEFAULT_BISECTION_THRESHOLD, DEFAULT_BISECTION_FACTOR
 from data_diff.joindiff_tables import TABLE_WRITE_LIMIT, JoinDiffer
-from data_diff.table_segment import TableSegment
-from data_diff.databases._connect import connect
 from data_diff.parse_time import parse_time_before, UNITS_STR, ParseError
-from data_diff.config import apply_config_from_file
+from data_diff.queries.api import current_timestamp
+from data_diff.schema import RawColumnInfo, create_schema
+from data_diff.table_segment import TableSegment
 from data_diff.tracking import disable_tracking, set_entrypoint_name
+from data_diff.utils import eval_name_template, remove_password_from_url, safezip, match_like, LogStatusHandler
 from data_diff.version import __version__
 
-
 COLOR_SCHEME = {
     "+": "green",
     "-": "red",
@@ -72,12 +70,12 @@ def _remove_passwords_in_dict(d: dict) -> None:
             d[k] = remove_password_from_url(v)
 
 
-def _get_schema(pair):
+def _get_schema(pair: Tuple[Database, DbPath]) -> Dict[str, RawColumnInfo]:
     db, table_path = pair
     return db.query_table_schema(table_path)
 
 
-def diff_schemas(table1, table2, schema1, schema2, columns):
+def diff_schemas(table1, table2, schema1, schema2, columns) -> None:
     logging.info("Diffing schemas...")
     attrs = "name", "type", "datetime_precision", "numeric_precision", "numeric_scale"
     for c in columns:
@@ -103,7 +101,7 @@ def diff_schemas(table1, table2, schema1, schema2, columns):
 
 
 class MyHelpFormatter(click.HelpFormatter):
-    def __init__(self, **kwargs):
+    def __init__(self, **kwargs) -> None:
         super().__init__(self, **kwargs)
         self.indent_increment = 6
 
@@ -281,7 +279,7 @@ click.Context.formatter_class = MyHelpFormatter
     default=None,
     help="Override the dbt production schema configuration within dbt_project.yml",
 )
-def main(conf, run, **kw):
+def main(conf, run, **kw) -> None:
     log_handlers = _get_log_handlers(kw["dbt"])
     if kw["table2"] is None and kw["database2"]:
         # Use the "database table table" form
@@ -341,14 +339,150 @@ def main(conf, run, **kw):
                 production_schema_flag=kw["prod_schema"],
             )
         else:
-            return _data_diff(
-                dbt_project_dir=project_dir_override, dbt_profiles_dir=profiles_dir_override, state=state, **kw
-            )
+            _data_diff(dbt_project_dir=project_dir_override, dbt_profiles_dir=profiles_dir_override, state=state, **kw)
     except Exception as e:
         logging.error(e)
         raise
 
 
+def _get_dbs(
+    threads: int, database1: str, threads1: int, database2: str, threads2: int, interactive: bool
+) -> Tuple[Database, Database]:
+    db1 = connect(database1, threads1 or threads)
+    if database1 == database2:
+        db2 = db1
+    else:
+        db2 = connect(database2, threads2 or threads)
+
+    if interactive:
+        db1.enable_interactive()
+        db2.enable_interactive()
+
+    return db1, db2
+
+
+def _set_age(options: dict, min_age: Optional[str], max_age: Optional[str], db: Database) -> None:
+    if min_age or max_age:
+        now: datetime = db.query(current_timestamp(), datetime).replace(tzinfo=None)
+        try:
+            if max_age:
+                options["min_update"] = parse_time_before(now, max_age)
+            if min_age:
+                options["max_update"] = parse_time_before(now, min_age)
+        except ParseError as e:
+            logging.error(f"Error while parsing age expression: {e}")
+
+
+def _get_table_differ(
+    algorithm: str,
+    db1: Database,
+    db2: Database,
+    threaded: bool,
+    threads: int,
+    assume_unique_key: bool,
+    sample_exclusive_rows: bool,
+    materialize_all_rows: bool,
+    table_write_limit: int,
+    materialize_to_table: Optional[str],
+    bisection_factor: Optional[int],
+    bisection_threshold: Optional[int],
+) -> TableDiffer:
+    algorithm = Algorithm(algorithm)
+    if algorithm == Algorithm.AUTO:
+        algorithm = Algorithm.JOINDIFF if db1 == db2 else Algorithm.HASHDIFF
+
+    logging.info(f"Using algorithm '{algorithm.name.lower()}'.")
+
+    if algorithm == Algorithm.JOINDIFF:
+        return JoinDiffer(
+            threaded=threaded,
+            max_threadpool_size=threads and threads * 2,
+            validate_unique_key=not assume_unique_key,
+            sample_exclusive_rows=sample_exclusive_rows,
+            materialize_all_rows=materialize_all_rows,
+            table_write_limit=table_write_limit,
+            materialize_to_table=(
+                materialize_to_table and db1.dialect.parse_table_name(eval_name_template(materialize_to_table))
+            ),
+        )
+
+    assert algorithm == Algorithm.HASHDIFF
+    return HashDiffer(
+        bisection_factor=DEFAULT_BISECTION_FACTOR if bisection_factor is None else bisection_factor,
+        bisection_threshold=DEFAULT_BISECTION_THRESHOLD if bisection_threshold is None else bisection_threshold,
+        threaded=threaded,
+        max_threadpool_size=threads and threads * 2,
+    )
+
+
+def _print_result(stats, json_output, diff_iter) -> None:
+    if stats:
+        if json_output:
+            rich.print(json.dumps(diff_iter.get_stats_dict()))
+        else:
+            rich.print(diff_iter.get_stats_string())
+
+    else:
+        for op, values in diff_iter:
+            color = COLOR_SCHEME.get(op, "grey62")
+
+            if json_output:
+                jsonl = json.dumps([op, list(values)])
+                rich.print(f"[{color}]{jsonl}[/{color}]")
+            else:
+                text = f"{op} {', '.join(map(str, values))}"
+                rich.print(f"[{color}]{text}[/{color}]")
+
+            sys.stdout.flush()
+
+
+def _get_expanded_columns(
+    columns: List[str],
+    case_sensitive: bool,
+    mutual: Set[str],
+    db1: Database,
+    schema1: dict,
+    table1: str,
+    db2: Database,
+    schema2: dict,
+    table2: str,
+) -> Set[str]:
+    expanded_columns: Set[str] = set()
+    for c in columns:
+        cc = c if case_sensitive else c.lower()
+        match = set(match_like(cc, mutual))
+        if not match:
+            m1 = None if any(match_like(cc, schema1.keys())) else f"{db1}/{table1}"
+            m2 = None if any(match_like(cc, schema2.keys())) else f"{db2}/{table2}"
+            not_matched = ", ".join(m for m in [m1, m2] if m)
+            raise ValueError(f"Column '{c}' not found in: {not_matched}")
+
+        expanded_columns |= match
+    return expanded_columns
+
+
+def _get_threads(threads: Union[int, str, None], threads1: Optional[int], threads2: Optional[int]) -> Tuple[bool, int]:
+    threaded = True
+    if threads is None:
+        threads = 1
+    elif isinstance(threads, str) and threads.lower() == "serial":
+        assert not (threads1 or threads2)
+        threaded = False
+        threads = 1
+    else:
+        try:
+            threads = int(threads)
+        except ValueError:
+            logging.error("Error: threads must be a number, or 'serial'.")
+            raise
+
+        if threads < 1:
+            logging.error("Error: threads must be >= 1")
+            raise ValueError("Error: threads must be >= 1")
+
+    return threaded, threads
+
+
 def _data_diff(
     database1,
     table1,
@@ -389,32 +523,13 @@ def _data_diff(
     threads1=None,
     threads2=None,
     __conf__=None,
-):
+) -> None:
     if limit and stats:
         logging.error("Cannot specify a limit when using the -s/--stats switch")
         return
 
     key_columns = key_columns or ("id",)
-    bisection_factor = DEFAULT_BISECTION_FACTOR if bisection_factor is None else int(bisection_factor)
-    bisection_threshold = DEFAULT_BISECTION_THRESHOLD if bisection_threshold is None else int(bisection_threshold)
-
-    threaded = True
-    if threads is None:
-        threads = 1
-    elif isinstance(threads, str) and threads.lower() == "serial":
-        assert not (threads1 or threads2)
-        threaded = False
-        threads = 1
-    else:
-        try:
-            threads = int(threads)
-        except ValueError:
-            logging.error("Error: threads must be a number, or 'serial'.")
-            return
-        if threads < 1:
-            logging.error("Error: threads must be >= 1")
-            return
-
+    threaded, threads = _get_threads(threads, threads1, threads2)
     start = time.monotonic()
 
     if database1 is None or database2 is None:
@@ -423,133 +538,79 @@ def _data_diff(
         )
         return
 
-    db1 = connect(database1, threads1 or threads)
-    if database1 == database2:
-        db2 = db1
-    else:
-        db2 = connect(database2, threads2 or threads)
+    db1: Database
+    db2: Database
+    db1, db2 = _get_dbs(threads, database1, threads1, database2, threads2, interactive)
+    with db1, db2:
+        options = {
+            "case_sensitive": case_sensitive,
+            "where": where,
+        }
 
-    options = dict(
-        case_sensitive=case_sensitive,
-        where=where,
-    )
+        _set_age(options, min_age, max_age, db1)
+        dbs: Tuple[Database, Database] = db1, db2
 
-    if min_age or max_age:
-        now: datetime = db1.query(current_timestamp(), datetime)
-        now = now.replace(tzinfo=None)
-        try:
-            if max_age:
-                options["min_update"] = parse_time_before(now, max_age)
-            if min_age:
-                options["max_update"] = parse_time_before(now, min_age)
-        except ParseError as e:
-            logging.error(f"Error while parsing age expression: {e}")
-            return
-
-    dbs: Tuple[Database, Database] = db1, db2
-
-    if interactive:
-        for db in dbs:
-            db.enable_interactive()
-
-    algorithm = Algorithm(algorithm)
-    if algorithm == Algorithm.AUTO:
-        algorithm = Algorithm.JOINDIFF if db1 == db2 else Algorithm.HASHDIFF
-
-    if algorithm == Algorithm.JOINDIFF:
-        differ = JoinDiffer(
-            threaded=threaded,
-            max_threadpool_size=threads and threads * 2,
-            validate_unique_key=not assume_unique_key,
-            sample_exclusive_rows=sample_exclusive_rows,
-            materialize_all_rows=materialize_all_rows,
-            table_write_limit=table_write_limit,
-            materialize_to_table=materialize_to_table
-            and db1.dialect.parse_table_name(eval_name_template(materialize_to_table)),
-        )
-    else:
-        assert algorithm == Algorithm.HASHDIFF
-        differ = HashDiffer(
-            bisection_factor=bisection_factor,
-            bisection_threshold=bisection_threshold,
-            threaded=threaded,
-            max_threadpool_size=threads and threads * 2,
+        differ = _get_table_differ(
+            algorithm,
+            db1,
+            db2,
+            threaded,
+            threads,
+            assume_unique_key,
+            sample_exclusive_rows,
+            materialize_all_rows,
+            table_write_limit,
+            materialize_to_table,
+            bisection_factor,
+            bisection_threshold,
         )
 
-    table_names = table1, table2
-    table_paths = [db.dialect.parse_table_name(t) for db, t in safezip(dbs, table_names)]
+        table_names = table1, table2
+        table_paths = [db.dialect.parse_table_name(t) for db, t in safezip(dbs, table_names)]
 
-    schemas = list(differ._thread_map(_get_schema, safezip(dbs, table_paths)))
-    schema1, schema2 = schemas = [
-        create_schema(db.name, table_path, schema, case_sensitive)
-        for db, table_path, schema in safezip(dbs, table_paths, schemas)
-    ]
+        schemas = list(differ._thread_map(_get_schema, safezip(dbs, table_paths)))
+        schema1, schema2 = schemas = [
+            create_schema(db.name, table_path, schema, case_sensitive)
+            for db, table_path, schema in safezip(dbs, table_paths, schemas)
+        ]
 
-    mutual = schema1.keys() & schema2.keys()  # Case-aware, according to case_sensitive
-    logging.debug(f"Available mutual columns: {mutual}")
+        mutual = schema1.keys() & schema2.keys()  # Case-aware, according to case_sensitive
+        logging.debug(f"Available mutual columns: {mutual}")
 
-    expanded_columns = set()
-    for c in columns:
-        cc = c if case_sensitive else c.lower()
-        match = set(match_like(cc, mutual))
-        if not match:
-            m1 = None if any(match_like(cc, schema1.keys())) else f"{db1}/{table1}"
-            m2 = None if any(match_like(cc, schema2.keys())) else f"{db2}/{table2}"
-            not_matched = ", ".join(m for m in [m1, m2] if m)
-            raise ValueError(f"Column '{c}' not found in: {not_matched}")
-
-        expanded_columns |= match
-
-    columns = tuple(expanded_columns - {*key_columns, update_column})
-
-    if db1 == db2:
-        diff_schemas(
-            table_names[0],
-            table_names[1],
-            schema1,
-            schema2,
-            (
-                *key_columns,
-                update_column,
-                *columns,
-            ),
+        expanded_columns = _get_expanded_columns(
+            columns, case_sensitive, mutual, db1, schema1, table1, db2, schema2, table2
         )
+        columns = tuple(expanded_columns - {*key_columns, update_column})
 
-    logging.info(f"Diffing using columns: key={key_columns} update={update_column} extra={columns}.")
-    logging.info(f"Using algorithm '{algorithm.name.lower()}'.")
+        if db1 == db2:
+            diff_schemas(
+                table_names[0],
+                table_names[1],
+                schema1,
+                schema2,
+                (
+                    *key_columns,
+                    update_column,
+                    *columns,
+                ),
+            )
 
-    segments = [
-        TableSegment(db, table_path, key_columns, update_column, columns, **options)._with_raw_schema(raw_schema)
-        for db, table_path, raw_schema in safezip(dbs, table_paths, schemas)
-    ]
+        logging.info(f"Diffing using columns: key={key_columns} update={update_column} extra={columns}.")
 
-    diff_iter = differ.diff_tables(*segments)
+        segments = [
+            TableSegment(db, table_path, key_columns, update_column, columns, **options)._with_raw_schema(raw_schema)
+            for db, table_path, raw_schema in safezip(dbs, table_paths, schemas)
+        ]
 
-    if limit:
-        assert not stats
-        diff_iter = islice(diff_iter, int(limit))
+        diff_iter = differ.diff_tables(*segments)
 
-    if stats:
-        if json_output:
-            rich.print(json.dumps(diff_iter.get_stats_dict()))
-        else:
-            rich.print(diff_iter.get_stats_string())
+        if limit:
+            assert not stats
+            diff_iter = islice(diff_iter, int(limit))
 
-    else:
-        for op, values in diff_iter:
-            color = COLOR_SCHEME.get(op, "grey62")
-
-            if json_output:
-                jsonl = json.dumps([op, list(values)])
-                rich.print(f"[{color}]{jsonl}[/{color}]")
-            else:
-                text = f"{op} {', '.join(map(str, values))}"
-                rich.print(f"[{color}]{text}[/{color}]")
-
-            sys.stdout.flush()
+        _print_result(stats, json_output, diff_iter)
 
     end = time.monotonic()
-
     logging.info(f"Duration: {end-start:.2f} seconds.")
 
 
