@@ -13,7 +13,7 @@ from uuid import uuid4
 import toml
 from rich import get_console
 
-from .version import __version__
+from data_diff.version import __version__
 
 TRACK_URL = "https://hosted.rudderlabs.com/v1/track"
 START_EVENT = "os_diff_run_start"
@@ -80,16 +80,16 @@ g_anonymous_id = None
 entrypoint_name = "Python API"
 
 
-def disable_tracking():
+def disable_tracking() -> None:
     global g_tracking_enabled
     g_tracking_enabled = False
 
 
-def is_tracking_enabled():
+def is_tracking_enabled() -> bool:
     return g_tracking_enabled
 
 
-def set_entrypoint_name(s):
+def set_entrypoint_name(s) -> None:
     global entrypoint_name
     entrypoint_name = s
 
@@ -99,22 +99,22 @@ dbt_version = None
 dbt_project_id = None
 
 
-def set_dbt_user_id(s):
+def set_dbt_user_id(s) -> None:
     global dbt_user_id
     dbt_user_id = s
 
 
-def set_dbt_version(s):
+def set_dbt_version(s) -> None:
     global dbt_version
     dbt_version = s
 
 
-def set_dbt_project_id(s):
+def set_dbt_project_id(s) -> None:
     global dbt_project_id
     dbt_project_id = s
 
 
-def get_anonymous_id():
+def get_anonymous_id() -> str:
     global g_anonymous_id
     if g_anonymous_id is None:
         profile = _load_profile()
@@ -201,10 +201,27 @@ def create_email_signup_event_json(email: str) -> Dict[str, Any]:
     }
 
 
-def send_event_json(event_json):
+def convert_sets_to_lists(obj):
+    """
+    Recursively convert sets in the given object to lists.
+    """
+    if isinstance(obj, set):
+        return list(obj)
+    elif isinstance(obj, dict):
+        return {k: convert_sets_to_lists(v) for k, v in obj.items()}
+    elif isinstance(obj, list):
+        return [convert_sets_to_lists(elem) for elem in obj]
+    else:
+        return obj
+
+
+def send_event_json(event_json) -> None:
     if not g_tracking_enabled:
         raise RuntimeError("Won't send; tracking is disabled!")
 
+    # Convert sets to lists in event_json
+    event_json = convert_sets_to_lists(event_json)
+
     headers = {
         "Content-Type": "application/json",
         "Authorization": "Basic MkhndE00SGNxOUJtZWlDcU5ZaHo3Tzl0a2pNOg==",
